@@ -12,26 +12,26 @@
 # language governing permissions and limitations under the License.
 import base64
 import binascii
-import json
 import hashlib
+import json
 import logging
 import re
 import sys
 import zlib
+from datetime import datetime, timedelta
 from zlib import error as ZLibError
-from datetime import timedelta
-from dateutil import tz, parser
 
-from pyasn1.error import PyAsn1Error
-import rsa
-
-from awscli.customizations.cloudtrail.utils import get_trail_by_arn, \
-    get_account_id_from_arn
-from awscli.customizations.commands import BasicCommand
+from awscrt.crypto import RSA, RSASignatureAlgorithm
 from botocore.exceptions import ClientError
-from awscli.compat import get_current_datetime
-from awscli.schema import ParameterRequiredError
-from awscli.utils import create_nested_client
+from dateutil import parser, tz
+
+from awscli.customizations.cloudtrail.utils import (
+    PublicKeyProvider,
+    get_account_id_from_arn,
+    get_trail_by_arn,
+)
+from awscli.customizations.commands import BasicCommand
+from awscli.customizations.exceptions import ParamValidationError
 
 LOG = logging.getLogger(__name__)
 DATE_FORMAT = '%Y%m%dT%H%M%SZ'
@@ -67,7 +67,9 @@ def parse_date(date_string):
     try:
         return parser.parse(date_string)
     except ValueError:
-        raise ValueError('Unable to parse date value: %s' % date_string)
+        raise ParamValidationError(
+            f'Unable to parse date value: {date_string}'
+        )
 
 
 def assert_cloudtrail_arn_is_valid(trail_arn):
@@ -76,7 +78,7 @@ def assert_cloudtrail_arn_is_valid(trail_arn):
     ARNs look like: arn:aws:cloudtrail:us-east-1:123456789012:trail/foo"""
     pattern = re.compile(r'arn:.+:cloudtrail:.+:\d{12}:trail/.+')
     if not pattern.match(trail_arn):
-        raise ValueError('Invalid trail ARN provided: %s' % trail_arn)
+        raise ParamValidationError(f'Invalid trail ARN provided: {trail_arn}')
 
 
 def create_digest_traverser(
@@ -138,11 +140,13 @@ def create_digest_traverser(
         is_org_trail = trail_info.get('IsOrganizationTrail')
         if is_org_trail:
             if not account_id:
-                raise ParameterRequiredError(
+                raise ParamValidationError(
                     "Missing required parameter for organization "
-                    "trail: '--account-id'")
+                    "trail: '--account-id'"
+                )
             organization_id = organization_client.describe_organization()[
-                'Organization']['Id']
+                'Organization'
+            ]['Id']
 
     # Determine the region from the ARN (e.g., arn:aws:cloudtrail:REGION:...)
     trail_region = trail_arn.split(':')[3]
@@ -153,24 +157,31 @@ def create_digest_traverser(
         account_id = get_account_id_from_arn(trail_arn)
 
     digest_provider = DigestProvider(
-        account_id=account_id, trail_name=trail_name,
+        account_id=account_id,
+        trail_name=trail_name,
         s3_client_provider=s3_client_provider,
         trail_source_region=trail_source_region,
         trail_home_region=trail_region,
-        organization_id=organization_id)
+        organization_id=organization_id,
+    )
     return DigestTraverser(
-        digest_provider=digest_provider, starting_bucket=bucket,
-        starting_prefix=prefix, on_invalid=on_invalid, on_gap=on_gap,
+        digest_provider=digest_provider,
+        starting_bucket=bucket,
+        starting_prefix=prefix,
+        on_invalid=on_invalid,
+        on_gap=on_gap,
         on_missing=on_missing,
-        public_key_provider=PublicKeyProvider(cloudtrail_client))
+        public_key_provider=PublicKeyProvider(cloudtrail_client),
+    )
 
 
-class S3ClientProvider(object):
+class S3ClientProvider:
     """Creates Amazon S3 clients and determines the region name of a client.
 
     This class will cache the location constraints of previously requested
     buckets and cache previously created clients for the same region.
     """
+
     def __init__(self, session, get_bucket_location_region='us-east-1'):
         self._session = session
         self._get_bucket_location_region = get_bucket_location_region
@@ -194,7 +205,7 @@ class S3ClientProvider(object):
     def _create_client(self, region_name):
         """Creates an Amazon S3 client for the given region name"""
         if region_name not in self._client_cache:
-            client = create_nested_client(self._session, 's3', region_name=region_name)
+            client = self._session.create_client('s3', region_name)
             # Remove the CLI error event that prevents exceptions.
             self._client_cache[region_name] = client
         return self._client_cache[region_name]
@@ -202,49 +213,30 @@ class S3ClientProvider(object):
 
 class DigestError(ValueError):
     """Exception raised when a digest fails to validate"""
+
     pass
 
 
 class DigestSignatureError(DigestError):
     """Exception raised when a digest signature is invalid"""
+
     def __init__(self, bucket, key):
-        message = ('Digest file\ts3://%s/%s\tINVALID: signature verification '
-                   'failed') % (bucket, key)
+        message = (
+            f'Digest file\ts3://{bucket}/{key}\tINVALID: signature verification '
+            'failed'
+        )
         super(DigestSignatureError, self).__init__(message)
 
 
 class InvalidDigestFormat(DigestError):
     """Exception raised when a digest has an invalid format"""
+
     def __init__(self, bucket, key):
-        message = 'Digest file\ts3://%s/%s\tINVALID: invalid format' % (bucket,
-                                                                        key)
+        message = f'Digest file\ts3://{bucket}/{key}\tINVALID: invalid format'
         super(InvalidDigestFormat, self).__init__(message)
 
 
-class PublicKeyProvider(object):
-    """Retrieves public keys from CloudTrail within a date range."""
-    def __init__(self, cloudtrail_client):
-        self._cloudtrail_client = cloudtrail_client
-
-    def get_public_keys(self, start_date, end_date):
-        """Loads public keys in a date range into a returned dict.
-
-        :type start_date: datetime
-        :param start_date: Start date of a date range.
-        :type end_date: datetime
-        :param end_date: End date of a date range.
-        :rtype: dict
-        :return: Returns a dict where each key is the fingerprint of the
-            public key, and each value is a dict of public key data.
-        """
-        public_keys = self._cloudtrail_client.list_public_keys(
-            StartTime=start_date, EndTime=end_date)
-        public_keys_in_range = public_keys['PublicKeyList']
-        LOG.debug('Loaded public keys in range: %s', public_keys_in_range)
-        return dict((key['Fingerprint'], key) for key in public_keys_in_range)
-
-
-class DigestProvider(object):
+class DigestProvider:
     """
     Retrieves digest keys and digests from Amazon S3.
 
@@ -290,7 +282,8 @@ class DigestProvider(object):
         target_start_date = format_date(normalize_date(start_date))
         # Add one hour to the end_date to get logs that spilled over to next.
         target_end_date = format_date(
-            normalize_date(end_date + timedelta(hours=1)))
+            normalize_date(end_date + timedelta(hours=1))
+        )
         # Ensure digests are from the same trail.
         digest_key_regex = re.compile(self._create_digest_key_regex(prefix))
         for key in key_filter:
@@ -312,19 +305,23 @@ class DigestProvider(object):
         client = self._client_provider.get_client(bucket)
         result = client.get_object(Bucket=bucket, Key=key)
         try:
-            digest = zlib.decompress(result['Body'].read(),
-                                     zlib.MAX_WBITS | 16)
+            digest = zlib.decompress(
+                result['Body'].read(), zlib.MAX_WBITS | 16
+            )
             digest_data = json.loads(digest.decode())
         except (ValueError, ZLibError):
             # Cannot gzip decode or JSON parse.
             raise InvalidDigestFormat(bucket, key)
         # Add the expected digest signature and algorithm to the dict.
-        if 'signature' not in result['Metadata'] \
-                or 'signature-algorithm' not in result['Metadata']:
+        if (
+            'signature' not in result['Metadata']
+            or 'signature-algorithm' not in result['Metadata']
+        ):
             raise DigestSignatureError(bucket, key)
         digest_data['_signature'] = result['Metadata']['signature']
-        digest_data['_signature_algorithm'] = \
-            result['Metadata']['signature-algorithm']
+        digest_data['_signature_algorithm'] = result['Metadata'][
+            'signature-algorithm'
+        ]
         return digest_data, digest
 
     def _create_digest_key(self, start_date, key_prefix):
@@ -346,7 +343,7 @@ class DigestProvider(object):
             'ymd': date.strftime('%Y/%m/%d'),
             'source_region': self.trail_source_region,
             'home_region': self.trail_home_region,
-            'name': self.trail_name
+            'name': self.trail_name,
         }
         if self.organization_id:
             template += '{organization_id}/'
@@ -368,7 +365,7 @@ class DigestProvider(object):
             'account_id': re.escape(self.account_id),
             'source_region': re.escape(self.trail_source_region),
             'home_region': re.escape(self.trail_home_region),
-            'name': re.escape(self.trail_name)
+            'name': re.escape(self.trail_name),
         }
         if self.organization_id:
             template += '{organization_id}/'
@@ -384,17 +381,31 @@ class DigestProvider(object):
         return '^' + key + '$'
 
 
-class DigestTraverser(object):
+class DigestTraverser:
     """Retrieves and validates digests within a date range."""
+
     # These keys are required to be present before validating the contents
     # of a digest.
-    required_digest_keys = ['digestPublicKeyFingerprint', 'digestS3Bucket',
-                            'digestS3Object', 'previousDigestSignature',
-                            'digestEndTime', 'digestStartTime']
+    required_digest_keys = [
+        'digestPublicKeyFingerprint',
+        'digestS3Bucket',
+        'digestS3Object',
+        'previousDigestSignature',
+        'digestEndTime',
+        'digestStartTime',
+    ]
 
-    def __init__(self, digest_provider, starting_bucket, starting_prefix,
-                 public_key_provider, digest_validator=None,
-                 on_invalid=None, on_gap=None, on_missing=None):
+    def __init__(
+        self,
+        digest_provider,
+        starting_bucket,
+        starting_prefix,
+        public_key_provider,
+        digest_validator=None,
+        on_invalid=None,
+        on_gap=None,
+        on_missing=None,
+    ):
         """
         :type digest_provider: DigestProvider
         :param digest_provider: DigestProvider object
@@ -433,7 +444,7 @@ class DigestTraverser(object):
         :param end_date: Date to stop validating at (inclusive).
         """
         if end_date is None:
-            end_date = get_current_datetime()
+            end_date = datetime.utcnow()
         end_date = normalize_date(end_date)
         start_date = normalize_date(start_date)
         bucket = self.starting_bucket
@@ -445,57 +456,91 @@ class DigestTraverser(object):
         while key and start_date <= last_start_date:
             try:
                 digest, end_date = self._load_and_validate_digest(
-                    public_keys, bucket, key)
+                    public_keys, bucket, key
+                )
                 last_start_date = normalize_date(
-                    parse_date(digest['digestStartTime']))
+                    parse_date(digest['digestStartTime'])
+                )
                 previous_bucket = digest.get('previousDigestS3Bucket', None)
                 yield digest
                 if previous_bucket is None:
                     # The chain is broken, so find next in digest store.
                     key, end_date = self._find_next_digest(
-                        digests=digests, bucket=bucket, last_key=key,
-                        last_start_date=last_start_date, cb=self._on_gap,
-                        is_cb_conditional=True)
+                        digests=digests,
+                        bucket=bucket,
+                        last_key=key,
+                        last_start_date=last_start_date,
+                        cb=self._on_gap,
+                        is_cb_conditional=True,
+                    )
                 else:
                     key = digest['previousDigestS3Object']
                     if previous_bucket != bucket:
                         bucket = previous_bucket
                         # The bucket changed so reload the digest list.
                         digests = self._load_digests(
-                            bucket, prefix, start_date, end_date)
+                            bucket, prefix, start_date, end_date
+                        )
             except ClientError as e:
                 if e.response['Error']['Code'] != 'NoSuchKey':
                     raise e
                 key, end_date = self._find_next_digest(
-                    digests=digests, bucket=bucket, last_key=key,
-                    last_start_date=last_start_date, cb=self._on_missing,
-                    message=str(e))
+                    digests=digests,
+                    bucket=bucket,
+                    last_key=key,
+                    last_start_date=last_start_date,
+                    cb=self._on_missing,
+                    message=str(e),
+                )
             except DigestError as e:
                 key, end_date = self._find_next_digest(
-                    digests=digests, bucket=bucket, last_key=key,
-                    last_start_date=last_start_date, cb=self._on_invalid,
-                    message=str(e))
+                    digests=digests,
+                    bucket=bucket,
+                    last_key=key,
+                    last_start_date=last_start_date,
+                    cb=self._on_invalid,
+                    message=str(e),
+                )
             except Exception as e:
                 # Any other unexpected errors.
                 key, end_date = self._find_next_digest(
-                    digests=digests, bucket=bucket, last_key=key,
-                    last_start_date=last_start_date, cb=self._on_invalid,
-                    message='Digest file\ts3://%s/%s\tINVALID: %s'
-                            % (bucket, key, str(e)))
+                    digests=digests,
+                    bucket=bucket,
+                    last_key=key,
+                    last_start_date=last_start_date,
+                    cb=self._on_invalid,
+                    message=f'Digest file\ts3://{bucket}/{key}\tINVALID: {str(e)}',
+                )
 
     def _load_digests(self, bucket, prefix, start_date, end_date):
         return self.digest_provider.load_digest_keys_in_range(
-            bucket=bucket, prefix=prefix,
-            start_date=start_date, end_date=end_date)
+            bucket=bucket,
+            prefix=prefix,
+            start_date=start_date,
+            end_date=end_date,
+        )
 
-    def _find_next_digest(self, digests, bucket, last_key, last_start_date,
-                          cb=None, is_cb_conditional=False, message=None):
+    def _find_next_digest(
+        self,
+        digests,
+        bucket,
+        last_key,
+        last_start_date,
+        cb=None,
+        is_cb_conditional=False,
+        message=None,
+    ):
         """Finds the next digest in the bucket and invokes any callback."""
         next_key, next_end_date = self._get_last_digest(digests, last_key)
         if cb and (not is_cb_conditional or next_key):
-            cb(bucket=bucket, next_key=next_key, last_key=last_key,
-               next_end_date=next_end_date, last_start_date=last_start_date,
-               message=message)
+            cb(
+                bucket=bucket,
+                next_key=next_key,
+                last_key=last_key,
+                next_end_date=next_end_date,
+                last_start_date=last_start_date,
+                message=message,
+            )
         return next_key, next_end_date
 
     def _get_last_digest(self, digests, before_key=None):
@@ -510,14 +555,16 @@ class DigestTraverser(object):
         elif before_key is None:
             next_key = digests.pop()
             next_key_date = normalize_date(
-                parse_date(extract_digest_key_date(next_key)))
+                parse_date(extract_digest_key_date(next_key))
+            )
             return next_key, next_key_date
         # find a key before the given key.
         before_key_date = parse_date(extract_digest_key_date(before_key))
         while digests:
             next_key = digests.pop()
             next_key_date = normalize_date(
-                parse_date(extract_digest_key_date(next_key)))
+                parse_date(extract_digest_key_date(next_key))
+            )
             if next_key_date < before_key_date:
                 LOG.debug("Next found key: %s", next_key)
                 return next_key, next_key_date
@@ -535,37 +582,40 @@ class DigestTraverser(object):
             if required_key not in digest_data:
                 raise InvalidDigestFormat(bucket, key)
         # Ensure the bucket and key are the same as what's expected.
-        if digest_data['digestS3Bucket'] != bucket \
-                or digest_data['digestS3Object'] != key:
+        if (
+            digest_data['digestS3Bucket'] != bucket
+            or digest_data['digestS3Object'] != key
+        ):
             raise DigestError(
-                ('Digest file\ts3://%s/%s\tINVALID: has been moved from its '
-                 'original location') % (bucket, key))
+                f'Digest file\ts3://{bucket}/{key}\tINVALID: has been moved from its '
+                'original location'
+            )
         # Get the public keys in the given time range.
         fingerprint = digest_data['digestPublicKeyFingerprint']
         if fingerprint not in public_keys:
             raise DigestError(
-                ('Digest file\ts3://%s/%s\tINVALID: public key not found in '
-                 'region %s for fingerprint %s') %
-                (bucket, key, self.digest_provider.trail_home_region,
-                 fingerprint))
+                f'Digest file\ts3://{bucket}/{key}\tINVALID: public key not found in '
+                f'region {self.digest_provider.trail_home_region} for fingerprint {fingerprint}'
+            )
         public_key_hex = public_keys[fingerprint]['Value']
         self._digest_validator.validate(
-            bucket, key, public_key_hex, digest_data, digest)
+            bucket, key, public_key_hex, digest_data, digest
+        )
         end_date = normalize_date(parse_date(digest_data['digestEndTime']))
         return digest_data, end_date
 
     def _load_public_keys(self, start_date, end_date):
         public_keys = self._public_key_provider.get_public_keys(
-            start_date, end_date)
+            start_date, end_date
+        )
         if not public_keys:
             raise RuntimeError(
-                'No public keys found between %s and %s' %
-                (format_display_date(start_date),
-                 format_display_date(end_date)))
+                f'No public keys found between {format_display_date(start_date)} and {format_display_date(end_date)}'
+            )
         return public_keys
 
 
-class Sha256RSADigestValidator(object):
+class Sha256RSADigestValidator:
     """
     Validates SHA256withRSA signed digests.
 
@@ -587,20 +637,29 @@ class Sha256RSADigestValidator(object):
         """
         try:
             decoded_key = base64.b64decode(public_key)
-            public_key = rsa.PublicKey.load_pkcs1(decoded_key, format='DER')
-            to_sign = self._create_string_to_sign(digest_data, inflated_digest)
-            signature_bytes = binascii.unhexlify(digest_data['_signature'])
-            rsa.verify(to_sign, signature_bytes, public_key)
-        except PyAsn1Error:
+            public_key = RSA.new_public_key_from_der_data(decoded_key)
+        except RuntimeError:
             raise DigestError(
-                ('Digest file\ts3://%s/%s\tINVALID: Unable to load PKCS #1 key'
-                 ' with fingerprint %s')
-                % (bucket, key, digest_data['digestPublicKeyFingerprint']))
-        except rsa.pkcs1.VerificationError:
-            # Note from the Python-RSA docs: Never display the stack trace of
-            # a rsa.pkcs1.VerificationError exception. It shows where in the
-            # code the exception occurred, and thus leaks information about
-            # the key.
+                (
+                    'Digest file\ts3://{}/{}\tINVALID: Unable to load PKCS #1 key'
+                    ' with fingerprint {}'
+                ).format(
+                    bucket, key, digest_data['digestPublicKeyFingerprint']
+                )
+            )
+
+        to_sign = self._create_string_to_sign(digest_data, inflated_digest)
+        signature_bytes = binascii.unhexlify(digest_data['_signature'])
+
+        result = public_key.verify(
+            signature_algorithm=RSASignatureAlgorithm.PKCS1_5_SHA256,
+            digest=hashlib.sha256(to_sign).digest(),
+            signature=signature_bytes,
+        )
+        if not result:
+            # The previous implementation caught a cryptography.exceptions.InvalidSignature
+            # exception here and re-raised the DigestSignatureError to avoid the stack trace
+            # leaking any information about the key.
             raise DigestSignatureError(bucket, key)
 
     def _create_string_to_sign(self, digest_data, inflated_digest):
@@ -608,12 +667,13 @@ class Sha256RSADigestValidator(object):
         if previous_signature is None:
             # The value must be 'null' to match the Java implementation.
             previous_signature = 'null'
-        string_to_sign = "%s\n%s/%s\n%s\n%s" % (
+        string_to_sign = "{}\n{}/{}\n{}\n{}".format(
             digest_data['digestEndTime'],
             digest_data['digestS3Bucket'],
             digest_data['digestS3Object'],
             hashlib.sha256(inflated_digest).hexdigest(),
-            previous_signature)
+            previous_signature,
+        )
         LOG.debug('Digest string to sign: %s', string_to_sign)
         return string_to_sign.encode()
 
@@ -622,6 +682,7 @@ class CloudTrailValidateLogs(BasicCommand):
     """
     Validates log digests and log files, optionally saving them to disk.
     """
+
     NAME = 'validate-logs'
     DESCRIPTION = """
     Validates CloudTrail logs for a given period of time.
@@ -668,34 +729,67 @@ class CloudTrailValidateLogs(BasicCommand):
     """
 
     ARG_TABLE = [
-        {'name': 'trail-arn', 'required': True, 'cli_type_name': 'string',
-         'help_text': 'Specifies the ARN of the trail to be validated'},
-        {'name': 'start-time', 'required': True, 'cli_type_name': 'string',
-         'help_text': ('Specifies that log files delivered on or after the '
-                       'specified UTC timestamp value will be validated. '
-                       'Example: "2015-01-08T05:21:42Z".')},
-        {'name': 'end-time', 'cli_type_name': 'string',
-         'help_text': ('Optionally specifies that log files delivered on or '
-                       'before the specified UTC timestamp value will be '
-                       'validated. The default value is the current time. '
-                       'Example: "2015-01-08T12:31:41Z".')},
-        {'name': 's3-bucket', 'cli_type_name': 'string',
-         'help_text': ('Optionally specifies the S3 bucket where the digest '
-                       'files are stored. If a bucket name is not specified, '
-                       'the CLI will retrieve it by calling describe_trails')},
-        {'name': 's3-prefix', 'cli_type_name': 'string',
-         'help_text': ('Optionally specifies the optional S3 prefix where the '
-                       'digest files are stored. If not specified, the CLI '
-                       'will determine the prefix automatically by calling '
-                       'describe_trails.')},
-        {'name': 'account-id', 'cli_type_name': 'string',
-         'help_text': ('Optionally specifies the account for validating logs. '
-                       'This parameter is needed for organization trails '
-                       'for validating logs for specific account inside an '
-                       'organization')},
-        {'name': 'verbose', 'cli_type_name': 'boolean',
-         'action': 'store_true',
-         'help_text': 'Display verbose log validation information'}
+        {
+            'name': 'trail-arn',
+            'required': True,
+            'cli_type_name': 'string',
+            'help_text': 'Specifies the ARN of the trail to be validated',
+        },
+        {
+            'name': 'start-time',
+            'required': True,
+            'cli_type_name': 'string',
+            'help_text': (
+                'Specifies that log files delivered on or after the '
+                'specified UTC timestamp value will be validated. '
+                'Example: "2015-01-08T05:21:42Z".'
+            ),
+        },
+        {
+            'name': 'end-time',
+            'cli_type_name': 'string',
+            'help_text': (
+                'Optionally specifies that log files delivered on or '
+                'before the specified UTC timestamp value will be '
+                'validated. The default value is the current time. '
+                'Example: "2015-01-08T12:31:41Z".'
+            ),
+        },
+        {
+            'name': 's3-bucket',
+            'cli_type_name': 'string',
+            'help_text': (
+                'Optionally specifies the S3 bucket where the digest '
+                'files are stored. If a bucket name is not specified, '
+                'the CLI will retrieve it by calling describe_trails'
+            ),
+        },
+        {
+            'name': 's3-prefix',
+            'cli_type_name': 'string',
+            'help_text': (
+                'Optionally specifies the optional S3 prefix where the '
+                'digest files are stored. If not specified, the CLI '
+                'will determine the prefix automatically by calling '
+                'describe_trails.'
+            ),
+        },
+        {
+            'name': 'account-id',
+            'cli_type_name': 'string',
+            'help_text': (
+                'Optionally specifies the account for validating logs. '
+                'This parameter is needed for organization trails '
+                'for validating logs for specific account inside an '
+                'organization'
+            ),
+        },
+        {
+            'name': 'verbose',
+            'cli_type_name': 'boolean',
+            'action': 'store_true',
+            'help_text': 'Display verbose log validation information',
+        },
     ]
 
     def __init__(self, session):
@@ -736,10 +830,12 @@ class CloudTrailValidateLogs(BasicCommand):
         if args.end_time:
             self.end_time = normalize_date(parse_date(args.end_time))
         else:
-            self.end_time = normalize_date(get_current_datetime())
+            self.end_time = normalize_date(datetime.utcnow())
         if self.start_time > self.end_time:
-            raise ValueError(('Invalid time range specified: start-time must '
-                              'occur before end-time'))
+            raise ParamValidationError(
+                'Invalid time range specified: start-time must '
+                'occur before end-time'
+            )
         # Found start time always defaults to the given start time. This value
         # may change if the earliest found digest is after the given start
         # time. Note that the summary output report of what date ranges were
@@ -751,26 +847,36 @@ class CloudTrailValidateLogs(BasicCommand):
         self._source_region = parsed_globals.region
         # Use the the same region as the region of the CLI to get locations.
         self.s3_client_provider = S3ClientProvider(
-            self._session, self._source_region)
-        client_args = {'region_name': parsed_globals.region,
-                       'verify': parsed_globals.verify_ssl}
-        self.organization_client = create_nested_client(
-            self._session, 'organizations', **client_args)
+            self._session, self._source_region
+        )
+        client_args = {
+            'region_name': parsed_globals.region,
+            'verify': parsed_globals.verify_ssl,
+        }
+        self.organization_client = self._session.create_client(
+            'organizations', **client_args
+        )
 
         if parsed_globals.endpoint_url is not None:
             client_args['endpoint_url'] = parsed_globals.endpoint_url
-        self.cloudtrail_client = create_nested_client(
-            self._session, 'cloudtrail', **client_args)
+        self.cloudtrail_client = self._session.create_client(
+            'cloudtrail', **client_args
+        )
 
     def _call(self):
         traverser = create_digest_traverser(
-            trail_arn=self.trail_arn, cloudtrail_client=self.cloudtrail_client,
+            trail_arn=self.trail_arn,
+            cloudtrail_client=self.cloudtrail_client,
             organization_client=self.organization_client,
             trail_source_region=self._source_region,
-            s3_client_provider=self.s3_client_provider, bucket=self.s3_bucket,
-            prefix=self.s3_prefix, on_missing=self._on_missing_digest,
-            on_invalid=self._on_invalid_digest, on_gap=self._on_digest_gap,
-            account_id=self.account_id)
+            s3_client_provider=self.s3_client_provider,
+            bucket=self.s3_bucket,
+            prefix=self.s3_prefix,
+            on_missing=self._on_missing_digest,
+            on_invalid=self._on_invalid_digest,
+            on_gap=self._on_digest_gap,
+            account_id=self.account_id,
+        )
         self._write_startup_text()
         digests = traverser.traverse(self.start_time, self.end_time)
         for digest in digests:
@@ -779,8 +885,10 @@ class CloudTrailValidateLogs(BasicCommand):
             self._track_found_times(digest)
             self._valid_digests += 1
             self._write_status(
-                'Digest file\ts3://%s/%s\tvalid'
-                % (digest['digestS3Bucket'], digest['digestS3Object']))
+                'Digest file\ts3://{}/{}\tvalid'.format(
+                    digest['digestS3Bucket'], digest['digestS3Object']
+                )
+            )
             if not digest['logFiles']:
                 continue
             for log in digest['logFiles']:
@@ -800,12 +908,13 @@ class CloudTrailValidateLogs(BasicCommand):
             self._found_end_time = min(digest_end_time, self.end_time)
 
     def _download_log(self, log):
-        """ Download a log, decompress, and compare SHA256 checksums"""
+        """Download a log, decompress, and compare SHA256 checksums"""
         try:
             # Create a client that can work with this bucket.
             client = self.s3_client_provider.get_client(log['s3Bucket'])
             response = client.get_object(
-                Bucket=log['s3Bucket'], Key=log['s3Object'])
+                Bucket=log['s3Bucket'], Key=log['s3Object']
+            )
             gzip_inflater = zlib.decompressobj(zlib.MAX_WBITS | 16)
             rolling_hash = hashlib.sha256()
             for chunk in iter(lambda: response['Body'].read(2048), b""):
@@ -819,8 +928,9 @@ class CloudTrailValidateLogs(BasicCommand):
                 self._on_log_invalid(log)
             else:
                 self._valid_logs += 1
-                self._write_status(('Log file\ts3://%s/%s\tvalid'
-                                    % (log['s3Bucket'], log['s3Object'])))
+                self._write_status(
+                    f"Log file\ts3://{log['s3Bucket']}/{log['s3Object']}\tvalid"
+                )
         except ClientError as e:
             if e.response['Error']['Code'] != 'NoSuchKey':
                 raise
@@ -831,35 +941,34 @@ class CloudTrailValidateLogs(BasicCommand):
     def _write_status(self, message, is_error=False):
         if is_error:
             if self._is_last_status_double_space:
-                sys.stderr.write("%s\n\n" % message)
+                sys.stderr.write(f"{message}\n\n")
             else:
-                sys.stderr.write("\n%s\n\n" % message)
+                sys.stderr.write(f"\n{message}\n\n")
             self._is_last_status_double_space = True
         elif self.is_verbose:
             self._is_last_status_double_space = False
-            sys.stdout.write("%s\n" % message)
+            sys.stdout.write(f"{message}\n")
 
     def _write_startup_text(self):
         sys.stdout.write(
-            'Validating log files for trail %s between %s and %s\n\n'
-            % (self.trail_arn, format_display_date(self.start_time),
-               format_display_date(self.end_time)))
+            f'Validating log files for trail {self.trail_arn} between {format_display_date(self.start_time)} and {format_display_date(self.end_time)}\n\n'
+        )
 
     def _write_summary_text(self):
         if not self._is_last_status_double_space:
             sys.stdout.write('\n')
-        sys.stdout.write('Results requested for %s to %s\n'
-                         % (format_display_date(self.start_time),
-                            format_display_date(self.end_time)))
+        sys.stdout.write(
+            f'Results requested for {format_display_date(self.start_time)} to {format_display_date(self.end_time)}\n'
+        )
         if not self._valid_digests and not self._invalid_digests:
             sys.stdout.write('No digests found\n')
             return
         if not self._found_start_time or not self._found_end_time:
             sys.stdout.write('No valid digests found in range\n')
         else:
-            sys.stdout.write('Results found for %s to %s:\n'
-                             % (format_display_date(self._found_start_time),
-                                format_display_date(self._found_end_time)))
+            sys.stdout.write(
+                f'Results found for {format_display_date(self._found_start_time)} to {format_display_date(self._found_end_time)}:\n'
+            )
         self._write_ratio(self._valid_digests, self._invalid_digests, 'digest')
         self._write_ratio(self._valid_logs, self._invalid_logs, 'log')
         sys.stdout.write('\n')
@@ -869,19 +978,25 @@ class CloudTrailValidateLogs(BasicCommand):
         if total > 0:
             sys.stdout.write('\n%d/%d %s files valid' % (valid, total, name))
             if invalid > 0:
-                sys.stdout.write(', %d/%d %s files INVALID' % (invalid, total,
-                                                               name))
+                sys.stdout.write(
+                    ', %d/%d %s files INVALID' % (invalid, total, name)
+                )
 
     def _on_missing_digest(self, bucket, last_key, **kwargs):
         self._invalid_digests += 1
-        self._write_status('Digest file\ts3://%s/%s\tINVALID: not found'
-                           % (bucket, last_key), True)
+        self._write_status(
+            f'Digest file\ts3://{bucket}/{last_key}\tINVALID: not found',
+            True,
+        )
 
     def _on_digest_gap(self, **kwargs):
         self._write_status(
-            'No log files were delivered by CloudTrail between %s and %s'
-            % (format_display_date(kwargs['next_end_date']),
-               format_display_date(kwargs['last_start_date'])), True)
+            'No log files were delivered by CloudTrail between {} and {}'.format(
+                format_display_date(kwargs['next_end_date']),
+                format_display_date(kwargs['last_start_date']),
+            ),
+            True,
+        )
 
     def _on_invalid_digest(self, message, **kwargs):
         self._invalid_digests += 1
@@ -890,17 +1005,28 @@ class CloudTrailValidateLogs(BasicCommand):
     def _on_invalid_log_format(self, log_data):
         self._invalid_logs += 1
         self._write_status(
-            ('Log file\ts3://%s/%s\tINVALID: invalid format'
-             % (log_data['s3Bucket'], log_data['s3Object'])), True)
+            (
+                'Log file\ts3://{}/{}\tINVALID: invalid format'.format(
+                    log_data['s3Bucket'], log_data['s3Object']
+                )
+            ),
+            True,
+        )
 
     def _on_log_invalid(self, log_data):
         self._invalid_logs += 1
         self._write_status(
-            "Log file\ts3://%s/%s\tINVALID: hash value doesn't match"
-            % (log_data['s3Bucket'], log_data['s3Object']), True)
+            "Log file\ts3://{}/{}\tINVALID: hash value doesn't match".format(
+                log_data['s3Bucket'], log_data['s3Object']
+            ),
+            True,
+        )
 
     def _on_missing_log(self, log_data):
         self._invalid_logs += 1
         self._write_status(
-            'Log file\ts3://%s/%s\tINVALID: not found'
-            % (log_data['s3Bucket'], log_data['s3Object']), True)
+            'Log file\ts3://{}/{}\tINVALID: not found'.format(
+                log_data['s3Bucket'], log_data['s3Object']
+            ),
+            True,
+        )
